--------------------------------------------------------------------------------
-- Animation Server definition -------------------------------------------------
-- you may not use this class directly. ----------------------------------------
--------------------------------------------------------------------------------
local AnimationServer = event.newObject(Object)
AnimationServer.animations = {}
setmetatable(AnimationServer.animations, {__mode = "v"})

function AnimationServer:routine()
    local yield = coroutine.yield
    while true do
        for k, a in pairs(self.animations) do a:next() end
        yield()
    end
end

--------------------------------------------------------------------------------
-- Animation definition --------------------------------------------------------
-- ivars -----------------------------------------------------------------------
--  frameTime: progress of current frame ---------------------------------------
--  frameNo: index of current frame --------------------------------------------
--  frames: list of frames, each describes duration and image handler ----------
--  finished: whether the animation finished, if it is not loop animation ------
--------------------------------------------------------------------------------

Animation = {}

function Animation.new(...)
    local obj = {frameTime = 1, frameNo = 1, frames = {...}}
    AnimationServer.animations[#AnimationServer.animations + 1] = obj
    return setmetatable(obj, {__index = Animation})
end

function Animation:copy()
    return Animation.new(unpack(self.frames))
end

function Animation:animate()
    return self.frames[self.frameNo].handler
end

function Animation:next()
    local frame = self.frames[self.frameNo]
    local ret = frame.handler
    
    self.frameTime = self.frameTime + 1
    if self.frameTime > frame.time then
        self.frameTime = 1
        self.frameNo = self.frameNo + 1
        if self.frameNo > #self.frames then self.frameNo = 1 end
        if self.frames[self.frameNo].time == 0 then
            self.finished = true

            -- self:next() should not do anything anymore.
            self.next = function(self) end
        end
    end
end

function Animation.keyframe(handler, time)
    return {handler = handler, time = time}
end

function Animation:drawAt(position)
    self:animate():drawAt(position)
end

function Animation:drawRotatedAt(position, angle)
    self:animate():drawRotatedAt(position, angle)
end

--[[ How to use

local f = Animation.keyframe
anim = Animation.new({f(img00, 4), f(img01, 10), f(nil, 0)})

anim:animate():drawAt(position)
if anim:finished then anim = nil end

--]]