Group = {
    routine = function(self)
        local yield = coroutine.yield
        while true do yield() end -- don't delete
    end
}

setmetatable(Group, {__index = Object})

function Group.new()
	local g = event.newObject(Group)
	g.group = Object.NONE
	g.members = {}
	g.memberCount = 0
	return g
end

function Group:add(obj)
    obj.finalize = function(this)
        self.memberCount = self.memberCount - 1
        if self.memberCount == 0 then self:delete() end
    end
    self.memberCount = self.memberCount + 1
	self.members[self.memberCount] = obj
end

function Group:randomMove(maxX, maxY, frame)
    local vs = {}
	for i, b in ipairs(self.members) do
	    vs[i] = vector.new(math.random(-maxX,maxX), math.random(-maxY,maxY))
	end
	for i = 1, frame do
	    for idx, b in ipairs(self.members) do b.position = b.position + vs[idx] end
	    coroutine.yield()
	end
end
function Group:forcemove(frame)
	local yield = coroutine.yield
	if frame ==-1 then
		while true do
			for i, b in ipairs(self.members) do
				b.position = b.position + b.velocity
			end
			yield()
		end
	end
	for i=1,frame do
		for i, b in ipairs(self.members) do
			b.position = b.position + b.velocity
		end
		yield()
	end
	return self
end

function Group:move(frame)
	wait(frame)
	return self
end

function Group:rotate(angle)
	for i,b in ipairs(self.members) do
	    b.velocity = b.velocity:rotate(angle)
	end
	return self
end
function Group:accelerate(acceleration)
	for i, b in ipairs(self.members) do
	    b.velocity:setLength(b.velocity:length() + acceleration)
	end
	return self
end
function Group:accelerateGradually(acceleration, frame)
	local yield = coroutine.yield
	local apf = acceleration / frame
	
	for i=1,frame do
	    for k, b in ipairs(self.members) do
	        b.velocity:setLength(b.velocity:length() + apf)
	    end
		yield()
	end
	return self
end
function Group:parallelize()
	local average_velocity = vector.new(0,0)
	for i,bullets in ipairs(self.members) do
		average_velocity = average_velocity + bullets.velocity
	end
	average_velocity = average_velocity / #self.members
	for i,bullets in ipairs(self.members) do
		bullets.velocity = average_velocity
	end
end

function Group:setVelocity(velocity)
	for i, b in ipairs(self.members) do
		b.velocity = velocity
	end
end
--̓ɂǁAgiveVelocityfollowerpB͕ʂɑx̂ɎgB
function Group:giveVelocity(velocity)
	for i, b in ipairs(self.members) do
		b.position = b.position + velocity
	end
end
function Group:addVelocity(velocity)
	for i, b in ipairs(self.members) do
		b.velocity = b.velocity + velocity
	end
end

function Group:bend(angle, frame)
	local yield = coroutine.yield
	local angular_velocity = angle / frame
	for i=1,frame do
		self:rotate(angular_velocity)
		yield()
	end
	return self
end
function Group:bendEach(angle,frame)
	local yield = coroutine.yield
	local angular_velocity = angle / frame
	for i, b in ipairs(self.members) do
		for j = 1, frame do
			b:rotate(angular_velocity)
			yield()
		end
	end
	return self
end
function Group:pause(frame)
	local table_temp = {}
	for i, b in ipairs(self.members) do
		table_temp[i] = b.velocity
		b.velocity = vector.new(0,0)
	end
	wait(frame)
	for i, b in ipairs(self.members) do
		b.velocity = table_temp[i]
	end
	return self
end
function Group:gather(target,speed)
	for i, b in ipairs(self.members) do
		b:gather(target,speed)
	end
	return self
end

function Group:update()
	for i,v in ipairs(self.members) do
		v.graphic:drawAt(v.position)
	end
end
