require("object")
require("vector")
require("animation")

local Player = {}
local Rin = {}
local Mika = {}

local Bomb = {}
local PenetrationBullet = {}
local BombBullet = {}
local PlayerShield = {}

Mika.partner = Rin
Mika.speed = 5
Mika.images = {}
Mika.shotcost = 100

Rin.partner = Mika
Rin.speed = 2.5
Rin.chargingTime = 0
Rin.images = {}
Rin.shotcost = 20

event.setShotCost = function(rin, mika)
	Rin.shotCost = rin
	Mika.shotCost = mika
end
event.addShotCost = function(rin, mika)
	Rin.shotCost	= Rin.shotCost + rin
	Mika.shotCost	= Mika.shotCost + mika
	if Rin.shotCost<10 then Rin.shotCost = 10 end
	if Mika.shotCost<50 then Mika.shotCost = 50 end
end

Bomb.images = graphics.loadTiled("graphics/system/bullet-b.png", 8, 1)
PenetrationBullet.image = graphics.load("graphics/system/bullet-p.png")
BombBullet.image = graphics.loadTiled("graphics/system/bullet-b.png", 8, 1)[1]

setmetatable(Player, {__index = Object})
setmetatable(Bomb, {__index = Object})
setmetatable(PenetrationBullet, {__index = Object})
setmetatable(BombBullet, {__index = Object})
setmetatable(PlayerShield, {__index = Object})

do -- make animations absurdly
    local f = Animation.keyframe
    local images = graphics.loadTiled("graphics/system/mika.png", 3, 3)
    Mika.images[1] = Animation.new(f(images[1], 20), f(images[2], 20), f(images[3], 20))
    Mika.images[2] = Animation.new(f(images[4], 20), f(images[5], 20), f(images[6], 20))
    Mika.images[3] = Animation.new(f(images[7], 20), f(images[8], 20), f(images[9], 20))
    images = graphics.loadTiled("graphics/system/rin.png", 3, 3)
    Rin.images[1] = Animation.new(f(images[1], 20), f(images[2], 20), f(images[3], 20))
    Rin.images[2] = Animation.new(f(images[4], 20), f(images[5], 20), f(images[6], 20))
    Rin.images[3] = Animation.new(f(images[7], 20), f(images[8], 20), f(images[9], 20))
end

--------------------------------------------------
--------------------------------------------------

function Player:initialize()
    graphics.heart = graphics.load("graphics/system/heart.png")
    self.position = vector.new(150, screen.h/2)
    self.frame.w = 8
    self.frame.h = 8
    self.group = Object.PLAYER
    self.state = Mika
    self.invisible_time = 0
    self.cooltime = 0
    self.canShotBullet = true
end

function Player:routine()
    local yield = coroutine.yield
    while true do
        local dx = (input.pressed(input.RIGHT) and 1 or 0) - (input.pressed(input.LEFT) and 1 or 0)
        local dy = (input.pressed(input.DOWN) and 1 or 0) - (input.pressed(input.UP) and 1 or 0)
        local m = (dx ~= 0 and dy ~= 0) and 1.0 / math.sqrt(2.0) or 1.0
        local speed = self.state.speed * m
        
        self.velocity = vector.new(dx, dy) * speed
        self.position = self.position + self.velocity

        if dx == 0 then self.spriteIndex = 1 end
        if dx > 0 then self.spriteIndex = 2 end
        if dx < 0 then self.spriteIndex = 3 end

        if self.frame.x < 0 then self.frame.x = 0 end
        if self.frame.y < 0 then self.frame.y = 0 end
        if self.frame.x + self.frame.w > screen.w then self.frame.x = screen.w - self.frame.w end
        if self.frame.y + self.frame.h > screen.h then self.frame.y = screen.h - self.frame.h end

        if self.canShotBullet then self.state:shotBullet() end
        if self.state:shouldSwitch() then self.state = self.state.partner end

        if self.invisible_time>0 then self.invisible_time = self.invisible_time - 1 end
		self.cooltime = self.cooltime - 1

        yield()
    end
end

function Player:draw()
	if self.invisible_time%6>0 then return end
	self.state:draw()
	graphics.heart:drawAt(self.position)
--	graphics.drawLineBox(self.frame, 0,0,0)
end

function Player:hit(obj)
	if self.invisible_time>0 then return end
    if obj.group == Object.ENEMY or obj.group == Object.ENEMY_SHIELD then
		score = score - 1000
        sound.beshot:play()
        createShot(self.position, vector.new(0,0), 0, PlayerShield)
        self.invisible_time = 180
    end
end

player = event.newObject(Player)

--------------------------------------------------
--------------------------------------------------

function Rin:shouldSwitch()
    return not input.pressed(input.B)
end

function Rin:draw()
    self.images[player.spriteIndex]:drawAt(player.position)
end

function Rin:shotBullet()
	if player.cooltime>0 then return end
    if input.pressed(input.A) then
        self.chargingTime = self.chargingTime + 1
    elseif self.chargingTime > 0 then
        local pb = event.newObject(PenetrationBullet)
        player.cooltime = 10
        pb.frame.x = player.frame.x + player.frame.w
        pb.position.y = player.position.y
        self.chargingTime = 0

        score = (score < self.shotcost) and 0 or score - self.shotcost
    end
end

--------------------------------------------------
--------------------------------------------------

function Mika:shouldSwitch()
    return input.pressed(input.B)
end

function Mika:draw()
    self.images[player.spriteIndex]:drawAt(player.position)
end

function Mika:shotBullet()
	if player.cooltime>0 then return end
    if (input.pushed(input.A)) then
        local bb = event.newObject(BombBullet)
        player.cooltime = 60
        bb.frame.x = player.frame.x + player.frame.w
        bb.position.y = player.position.y

        score = (score < self.shotcost) and 0 or score - self.shotcost
    end
end

--------------------------------------------------
--------------------------------------------------

function Bomb:initialize()
    self.frame.w = 64
    self.frame.h = 64
    self.group = Object.PLAYER_BULLET
    self.time = 60

    local f = Animation.keyframe
    self.image = Animation.new(f(Bomb.images[2], 1), f(Bomb.images[3], 1), f(Bomb.images[4], 1), f(Bomb.images[5], 1), f(Bomb.images[6], 1), f(Bomb.images[7], 1), f(Bomb.images[8], 0))
end

function Bomb:routine()
    local yield = coroutine.yield
    while self.time > 0 do
        self.time = self.time - 1
        yield()
    end
end

function Bomb:draw()
    self.image:drawAt(self.position)
end

--------------------------------------------------
--------------------------------------------------

function PlayerShield:initialize()
    self.group = Object.PLAYER_BULLET
    self.frame.w = 300
    self.frame.h = 300
end

function PlayerShield:routine()
    wait(1)
end
--------------------------------------------------
--------------------------------------------------

function BombBullet:initialize()
    self.frame.w = 8
    self.frame.h = 8
    self.group = Object.PLAYER_BULLET
end

function BombBullet:routine()
    local yield = coroutine.yield
    while true do
        self.position.x = self.position.x + 12
        
        if self.frame.x > screen.w then self:explode() end
        if not input.pressed(input.A) then self:explode() end
        yield()
    end
end

function BombBullet:draw()
    BombBullet.image:drawAt(self.position)
end

function BombBullet:hit(obj)
    if obj.group == Object.ENEMY or obj.group == Object.ENEMY_SHIELD then
        self:explode()
    end
end

function BombBullet:explode()
    local b = event.newObject(Bomb)
    b.position = self.position
    self:delete()
end

--------------------------------------------------
--------------------------------------------------

function PenetrationBullet:initialize()
    self.frame.w = 64
    self.frame.h = 8
    self.group = Object.PLAYER_BULLET
end

function PenetrationBullet:routine()
    local yield = coroutine.yield
    while true do
        self.position.x = self.position.x + 16
        yield()
    end
end

function PenetrationBullet:draw()
    self.image:drawAt(self.position)
end

function PenetrationBullet:hit(object)
    if (object.group == Object.ENEMY_SHIELD) then self:delete() end
end

--------------------------------------------------
--------------------------------------------------


